'Programm:   Demo fr den UKW-Fuchsjagd-Sender "Foxy" mit ATtiny45+MICRF102
'            Morseton mit mit Timer1 erzeugen und als ASK senden, Morsetaste an Pin MISO/INT0/PB2 nach Masse.
'            Der Ton wird so lange gesendet, wie die Taste gedrckt ist. Das Programm entstand auf Wunsch
'            eines einzelnen Herrn 8-)
'Sprache:    Bascom-AVR-Basic, siehe www.mcselec.com (freie Version hat fr ATtiny45 keine Einschrnkungen)
'Von:        Roland Walter, DL7UNO, www.rowalt.de
'Datum:      31.01.2007 (letzte nderung)
'Literatur:  Roland Walter: AVR-Mikrocontroller-Lehrbuch, siehe www.rowalt.de
'Kommentar:  Pin PB4 wird als Timer1-Ausgang OC1B verwendet, Timer1 erzeugt den
'            Ton selbstndig. Etwa 9% des Programmspeichers werden verwendet...
'            ber Pin PB2/INT0 wird ein externer Interrupt ausgelst, der den AVR aus dem Sleep-Modus
'            aufweckt. Ansonsten schlft der AVR nach etwa 5sec Inaktivitt ein. Im Sleepzustand ist
'            die Gesamtstromaufnahme der Schaltung etwa 250...300 Mikroampere. Eventuell kann man da
'            an einigen Ecken noch mehr herausholen, aber fr den Demo-Zweck soll das erst einmal reichen.
'-------------------------------------------------------------------------------
$regfile = "ATtiny45.dat"   'Deklarationen fr den ATtiny45
$crystal = 1000000          'Taktfrequenz: 1 MHz (Fabrik-Voreinstellung, Interner RC-Oszillator 8MHz mitb Vorteilung 1/8)
'
Dim lSleepCounter As Long   'Zhlt hoch, wann wir nach dem letzten Tastendruck schlafen gehen knnen
'
On INT0 OnExternalInterrupt 'Interrupt-Routine fr externen Interrupt 0 deklarieren
'
ACSR.ACD = 0                'Analog-Komparator ausschalten, spart etwas Strom
'
DDRB  = &B00001000          'Pins PB3 als Ausgang konfigurieren (1), Rest ist Eingang (0). PB4 wird "dynamisch" zum Ausgang
PORTB = &B00000111          'PullUp-Widerstnde fr die Pins PB0,PB1,PB2 aktivieren (PB2: Morsetaste,PB0+PB1: Strfestigkeit)
'
'Statt den Ton zu Fu durch Ein- und Ausschalten von Pin PB4 zu erzeugen, konfigurieren
'wir jetzt Timer1 so, da er dies selbstndig und unabhngig vom laufenden Programm fr
'uns tut. Sobald wir die entsprechenden 4 Timer1-Register beschickt haben, beginnt die
'Tonausgabe:
TCCR1=Bits(CS11,CS10,CTC1) 'Timer1: Vorteilung 1MHz/8, Timer-Reset bei Compare-Match
GTCCR=Bits(COM1B0)         'Timer1: Toggle Pin OC1B/PB4, wenn Register TCNT1 Wert von Compare-Register OCR1B hat
OCR1B=150                  'Timer1: Ausprobierter Vergleichswert. Wenn erreicht, dann Aktion an Pin OC1B/PB4
OCR1C=OCR1B                'Timer1: Timer-Maximalwert wie Vergleichswert setzen
'
GIMSK.INT0=1               'Externen Interrupt 0 (ber Pin INT0/PB2) einschalten
SREG.SREG_I=1              'Global Interrupt Enable
'
Do
  If PINB.2=0 Then  'Wenn Morsetaste gedrckt (Pin PB2 nach Gnd gezogen)
    PORTB.3=1       'Pin PB3 auf High (StandBy-Pin am MICRF102): Standby-Modus abschalten, falls noch aktiv
    DDRB.4=1        'Pin PB4 zum Ausgang machen: Tonsignal von Timer1 gelangt jetzt nach auen
    lSleepCounter=0 'Einschlaf-Counter zurcksetzen
  Else
    DDRB.4=0            'Pin PB4 zum Eingang machen: Tonsignal von Timer1 gelangt nicht nach auen
    Incr lSleepCounter  'Wieder ein Schritt nher am Einschlafen
  End If
  If lSleepCounter>65000 Then  'Einschlaf-Zhlerstand erreicht (65000 entspricht etwa 5sec, Wert wurde einfach ausprobiert)
    lSleepCounter=0
    PORTB.3=0       'Pin PB3 auf High (StandBy-Pin am MICRF102): In den Standby-Modus gehen
    PowerDown
  End If
Loop
'-------------------------------------------------------------------------------
OnExternalInterrupt:  'Interrupt-Routine fr externen Interrupt 0
Return
'-------------------------------------------------------------------------------
End